<?php
namespace plugins\antispam\library;

/**
 * InvisibleCaptcha class
 * @author Jérôme Brilland
 * @version 1.0
 * @copyright (c) 2023, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class InvisibleCaptcha {
    
    /**
     * @param string form Form name
     * @return string Token
     * @access public
     */
    public function generateToken( string $formName ) {
        $token = hash( 'sha512', $formName . uniqid( microtime(), true ) . $_SERVER['REMOTE_ADDR'] . bin2hex( random_bytes(32) ) );
        $_SESSION[$formName . '_token'] = $token;
        return $token;
    }
    
    /**
     * @param string form Form name
     * @return boolean
     * @access public
     */
    public function checkToken( $formName, $token ) {
        if ( !isset( $_SESSION[$formName . '_token'] ) ) {
            return false;
        }
        if ( $_SESSION[$formName . '_token'] !== $token ) {
            return false;
        }
        unset( $_SESSION[$formName . '_token'] );
        return true;
    }
    
    /**
     * @access public
     */
    public function createTimetrap() {
        $_SESSION['antispam_timestamp'] = time();
    } 
    
    /**
     * @param int minimumTime Minimum time
     * @return boolean
     * @access public
     */
    public function checkTimetrap( int $minimumTime = 30 ) {
        $timestamp = $_SESSION['antispam_timestamp'];
        $now = time();
        if ( empty( $timestamp ) ) {
            return false;
        }
        //timestamp is too old
        if ( $timestamp < $now - 43200 ) {
            return false;
        }
        unset( $_SESSION['antispam_timestamp'] );
        return ( $now - $timestamp < $minimumTime );
    }
    
}
